// TMCL.cpp : Communicate with a TMCM module in TMCL. 
// Uses code from TMCLTest.cpp (www.trinamic.de) 
//

#include "stdafx.h"
#include <stdio.h>
#include <stdlib.h>
#include <windows.h>

#include "tmcl.h"

//Opcodes of all TMCL commands that can be used in direct mode
#define TMCL_ROR 1
#define TMCL_ROL 2
#define TMCL_MST 3
#define TMCL_MVP 4
#define TMCL_SAP 5
#define TMCL_GAP 6
#define TMCL_STAP 7
#define TMCL_RSAP 8
#define TMCL_SGP 9
#define TMCL_GGP 10
#define TMCL_STGP 11
#define TMCL_RSGP 12
#define TMCL_RFS 13
#define TMCL_SIO 14
#define TMCL_GIO 15
#define TMCL_SCO 30
#define TMCL_GCO 31
#define TMCL_CCO 32

//Opcodes of TMCL control functions (to be used to run or abort a TMCL program in the module)
#define TMCL_APPL_STOP 128
#define TMCL_APPL_RUN 129
#define TMCL_APPL_RESET 131

//Options for MVP commandds
#define MVP_ABS 0
#define MVP_REL 1
#define MVP_COORD 2

//Options for RFS command
#define RFS_START 0
#define RFS_STOP 1
#define RFS_STATUS 2

#define FALSE 0
#define TRUE 1

//Result codes for GetResult
#define TMCL_RESULT_OK 0
#define TMCL_RESULT_NOT_READY 1
#define TMCL_RESULT_CHECKSUM_ERROR 2


//Open serial interface
//Usage: ComHandle=OpenRS232("COM1", CBR_9600)
HANDLE OpenRS232(const char* ComName, DWORD BaudRate)
{
	HANDLE ComHandle;
	DCB CommDCB;
	COMMTIMEOUTS CommTimeouts;

	ComHandle=CreateFileA(ComName, GENERIC_READ|GENERIC_WRITE, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);

	if(GetLastError()!=ERROR_SUCCESS) return INVALID_HANDLE_VALUE;
	else
	{
		GetCommState(ComHandle, &CommDCB);

		CommDCB.BaudRate=BaudRate;
		CommDCB.Parity=NOPARITY;
		CommDCB.StopBits=ONESTOPBIT;
		CommDCB.ByteSize=8;

		CommDCB.fBinary=1;  //Binary Mode only
		CommDCB.fParity=0;
		CommDCB.fOutxCtsFlow=0;
		CommDCB.fOutxDsrFlow=0;
		CommDCB.fDtrControl=0;
		CommDCB.fDsrSensitivity=0;
		CommDCB.fTXContinueOnXoff=0;
		CommDCB.fOutX=0;
		CommDCB.fInX=0;
		CommDCB.fErrorChar=0;
		CommDCB.fNull=0;
		CommDCB.fRtsControl=RTS_CONTROL_TOGGLE;
		CommDCB.fAbortOnError=0;

		SetCommState(ComHandle, &CommDCB);

		//Set buffer size
		SetupComm(ComHandle, 100, 100);

		//Set up timeout values (very important, as otherwise the program will be very slow)
		GetCommTimeouts(ComHandle, &CommTimeouts);

		CommTimeouts.ReadIntervalTimeout=MAXDWORD;
		CommTimeouts.ReadTotalTimeoutMultiplier=0;
		CommTimeouts.ReadTotalTimeoutConstant=0;

		SetCommTimeouts(ComHandle, &CommTimeouts);

		return ComHandle;
  }
}


//Close the serial port
//Usage: CloseRS232(ComHandle);
void CloseRS232(HANDLE Handle)
{
	CloseHandle(Handle);
}


//Send a binary TMCL command
//e.g.  SendCmd(ComHandle, 1, TMCL_MVP, MVP_ABS, 1, 50000);   will be MVP ABS, 1, 50000 for a module with address 1
//Parameters: Handle: Handle of the serial port (returned by OpenRS232).
//            Address: address of the module (factory default is 1).
//            Command: the TMCL command (see the constants at the begiining of this file)
//            Type:    the "Type" parameter of the TMCL command (set to 0 if unused)
//            Motor:   the motor number (set to 0 if unused)
//            Value:   the "Value" parameter (depending on the command, set to 0 if unused)
void SendCmd(HANDLE Handle, UCHAR Address, UCHAR Command, UCHAR Type, UCHAR Motor, INT Value)
{
	UCHAR TxBuffer[9];
	DWORD BytesWritten;
	int i;

	TxBuffer[0]=Address;
	TxBuffer[1]=Command;
	TxBuffer[2]=Type;
	TxBuffer[3]=Motor;
	TxBuffer[4]=Value >> 24;
	TxBuffer[5]=Value >> 16;
	TxBuffer[6]=Value >> 8;
	TxBuffer[7]=Value & 0xff;
	TxBuffer[8]=0;
	for(i=0; i<8; i++)
		TxBuffer[8]+=TxBuffer[i];

	//Send the datagram
	WriteFile(Handle, TxBuffer, 9, &BytesWritten, NULL);
}


//Read the result that is returned by the module
//Parameters: Handle: handle of the serial port, as returned by OpenRS232
//            Address: pointer to variable to hold the reply address returned by the module
//            Status: pointer to variable to hold the status returned by the module (100 means okay)
//            Value: pointer to variable to hold the value returned by the module
//Return value: TMCL_RESULT_OK: result has been read without errors
//              TMCL_RESULT_NOT_READY: not enough bytes read so far (try again)
//              TMCL_RESULT_CHECKSUM_ERROR: checksum of reply packet wrong
UCHAR GetResult(HANDLE Handle, UCHAR *Address, UCHAR *Status, int *Value)
{
	UCHAR RxBuffer[9], Checksum;
	DWORD Errors, BytesRead;
	COMSTAT ComStat;
	int i;

	//Check if enough bytes can be read
	ClearCommError(Handle, &Errors, &ComStat);
    if(ComStat.cbInQue>8)
	{
		//Receive
		ReadFile(Handle, RxBuffer, 9, &BytesRead, NULL);

		Checksum=0;
		for(i=0; i<8; i++)
			Checksum+=RxBuffer[i];

		if(Checksum!=RxBuffer[8]) return TMCL_RESULT_CHECKSUM_ERROR;

		*Address=RxBuffer[0];
		*Status=RxBuffer[2];
		*Value=(RxBuffer[4] << 24) | (RxBuffer[5] << 16) | (RxBuffer[6] << 8) | RxBuffer[7];
	} else return TMCL_RESULT_NOT_READY;

	return TMCL_RESULT_OK;
}

int tmclCmd(HANDLE Handle, UCHAR Address, UCHAR Command, UCHAR Type, UCHAR Motor, INT Value)
{
	UCHAR ResultAddress, ResultStatus;
	int Result, ResultValue, Timeout;

	SendCmd(Handle, Address, Command, Type, Motor, Value);

	ResultAddress=0;
	ResultStatus=0;
	ResultValue=0;
	Timeout=GetTickCount();
	while((Result = GetResult(Handle, &ResultAddress, &ResultStatus, &ResultValue))==TMCL_RESULT_NOT_READY && abs((int)(GetTickCount()-Timeout))<1000);
	
	if ((Result == TMCL_RESULT_OK) && (ResultAddress == 2) && (ResultStatus == 100))
		return ResultValue;
    else if (Result == TMCL_RESULT_OK)
    {
		printf("\nTMCL Unexpected result Address %d, Status %d, Value %d\n", ResultAddress, ResultStatus, ResultValue);
        exit(-1);
    }
    else if (Result == TMCL_RESULT_CHECKSUM_ERROR) 
	{
		printf("\nTMCL Checksum wrong\n");
        exit(-1);
    }
    else if (Result == TMCL_RESULT_NOT_READY) 
	{
		printf("\nTMCL Timeout\n");
        exit(-1);
    }
    else
	{
		printf("\nTMCL Error\n");
        exit(-1);
    }

	return 0;
}


int tmclMove(char* commport, int offset)
{
	int Value;
	int Position, NewPosition;

//	printf("commport: %s offset: %d \n", commport, offset);

	HANDLE RS232Handle;

	RS232Handle=OpenRS232(commport, 9600);
	if (RS232Handle == INVALID_HANDLE_VALUE)
    {
		printf("Error: Could not open comm port %s\n", commport);
        exit(-1);
    }
	Position = offset;
    if ((Position < -8388608) || (Position > 8388607))
    {
		printf("Error: Offset out of range\n");
        exit(-1);
    }
	
    /* Move Relative */
	NewPosition = tmclCmd(RS232Handle, 1, TMCL_MVP, 1, 0, Position);

	/* Poll whether position reached */
	Value = 0;
	while (Value == 0)
	{
		/* Wait one second */
		Sleep(1000);
		printf (".");

		/* Inquire Motor Position Reached */
		Value = tmclCmd(RS232Handle, 1, TMCL_GAP, 8, 0, 0);		
	}
	printf ("\nposition %d\n", NewPosition);

	CloseRS232(RS232Handle);

	return 0;

}

